/**
 * OW2 Util
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: RunnableHelper.java 4389 2008-12-15 13:48:57Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.execution.helper;

import java.security.AccessController;
import java.security.PrivilegedAction;

import org.ow2.util.execution.ExecutionResult;
import org.ow2.util.execution.IExecution;
import org.ow2.util.execution.IRunner;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Helps to run blocks of code under a controlled environment.
 * @author Guillaume Sauthier
 * @param <T> execution result type.
 */
public final class RunnableHelper<T> implements IRunner<T> {

    /**
     * Logger used.
     */
    private Log logger = LogFactory.getLog(RunnableHelper.class);


    /**
     * @return Return the current {@link Thread} context ClassLoader
     *         (using a privileged block).
     */
    private static ClassLoader getThreadContextClassLoader() {
        return AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            public ClassLoader run() {
                return Thread.currentThread().getContextClassLoader();
            }
        });
    }

    /**
     * Run a code block in the context of a given ClassLoader.
     * @param loader The {@link ClassLoader} to be used as TCCL.
     * @param exec code block.
     * @return the result of this execution.
     */
    public ExecutionResult<T> execute(final ClassLoader loader, final IExecution<T> exec) {
        ExecutionResult<T> result = new ExecutionResult<T>();

        // Store the old loader
        ClassLoader old = getThreadContextClassLoader();
        try {
            // Set the new one
            Thread.currentThread().setContextClassLoader(loader);

            // Run the code and store the result
            result.setResult(exec.execute());
        } catch (Exception e) {
            // Log
            logger.debug("IExecution produced Exception: {0}", e.getMessage());
            // Store
            result.setException(e);
        } finally {
            // Revert to the previous loader
            Thread.currentThread().setContextClassLoader(old);
        }

        // Return the result holder
        return result;
    }
}
